@@ -1,5 +1,5 @@
 # Enthought library imports.
-from traits.api import Callable, HasTraits, Str, Trait
+from traits.api import Callable, HasTraits, Str, Enum
 
 
 class SchemaAddition(HasTraits):
@@ -33,3 +33,13 @@ class SchemaAddition(HasTraits):
     # - for groups, this is the ID of another group.
     # - for menus and actions, this is the ID of another menu or action.
     before = Str
+
+    # The action appears at the absolute specified position first or last.
+    # This is useful for example to keep the File menu the first menu
+    # in a menubar, the help menu the last etc.
+    # If multiple actions in a schema have absolute_position 'first',
+    # they will appear in the same order specified; likewise for 'last'.
+    # Absolute positions are subject to subsequent topological reordering
+    # if 'after' or 'before' traits are also specified, hence it must not
+    # be used alongwith after and before traits
+    absolute_position = Enum(None, 'first', 'last')
