@@ -46,6 +46,9 @@ class _MenuItem(HasTraits):
     # a group).
     group = Any
 
+    # The toolkit control.
+    control = Any()
+
     ###########################################################################
     # 'object' interface.
     ###########################################################################
@@ -99,6 +102,11 @@ class _MenuItem(HasTraits):
         action.on_trait_change(self._on_action_visible_changed, 'visible')
         action.on_trait_change(self._on_action_checked_changed, 'checked')
         action.on_trait_change(self._on_action_name_changed, 'name')
+        action.on_trait_change(self._on_action_accelerator_changed,
+                               'accelerator')
+
+        # Detect if the control is destroyed.
+        self.control.destroyed.connect(self._qt4_on_destroyed)
 
         if controller is not None:
             self.controller = controller
@@ -114,11 +122,19 @@ class _MenuItem(HasTraits):
             remove=True)
         action.on_trait_change(self._on_action_name_changed, 'name',
             remove=True)
+        action.on_trait_change(self._on_action_accelerator_changed,
+            'accelerator', remove=True)
 
     ###########################################################################
     # Private interface.
     ###########################################################################
 
+    def _qt4_on_destroyed(self, control=None):
+        """ Delete the reference to the control to avoid attempting to talk to
+        it again.
+        """
+        self.control = None
+
     def _qt4_on_triggered(self):
         """ Called when the menu item has been clicked. """
 
@@ -130,12 +146,7 @@ class _MenuItem(HasTraits):
         # Perform the action!
         if self.controller is not None:
             if is_checkable:
-                # fixme: There is a difference here between having a controller
-                # and not in that in this case we do not set the checked state
-                # of the action! This is confusing if you start off without a
-                # controller and then set one as the action now behaves
-                # differently!
-                self.checked = self.control.isChecked()
+                self.checked = action.checked = self.control.isChecked()
 
             # Most of the time, action's do no care about the event (it
             # contains information about the time the event occurred etc), so
@@ -153,7 +164,7 @@ class _MenuItem(HasTraits):
 
         else:
             if is_checkable:
-                action.checked = self.control.isChecked()
+                self.checked = action.checked = self.control.isChecked()
 
             # Most of the time, action's do no care about the event (it
             # contains information about the time the event occurred etc), so
@@ -171,38 +182,43 @@ class _MenuItem(HasTraits):
 
     def _enabled_changed(self):
         """ Called when our 'enabled' trait is changed. """
-
-        self.control.setEnabled(self.enabled)
+        if self.control is not None:
+            self.control.setEnabled(self.enabled)
 
     def _visible_changed(self):
         """ Called when our 'visible' trait is changed. """
-
-        self.control.setVisible(self.visible)
+        if self.control is not None:
+            self.control.setVisible(self.visible)
 
     def _checked_changed(self):
         """ Called when our 'checked' trait is changed. """
-
-        self.control.setChecked(self.checked)
+        if self.control is not None:
+            self.control.setChecked(self.checked)
 
     def _on_action_enabled_changed(self, action, trait_name, old, new):
         """ Called when the enabled trait is changed on an action. """
-
-        self.control.setEnabled(action.enabled)
+        if self.control is not None:
+            self.control.setEnabled(action.enabled)
 
     def _on_action_visible_changed(self, action, trait_name, old, new):
         """ Called when the visible trait is changed on an action. """
-
-        self.control.setVisible(action.visible)
+        if self.control is not None:
+            self.control.setVisible(action.visible)
 
     def _on_action_checked_changed(self, action, trait_name, old, new):
         """ Called when the checked trait is changed on an action. """
-
-        self.control.setChecked(action.checked)
+        if self.control is not None:
+            self.control.setChecked(action.checked)
 
     def _on_action_name_changed(self, action, trait_name, old, new):
         """ Called when the name trait is changed on an action. """
+        if self.control is not None:
+            self.control.setText(action.name)
 
-        self.control.setText(action.name)
+    def _on_action_accelerator_changed(self, action, trait_name, old, new):
+        """ Called when the accelerator trait is changed on an action. """
+        if self.control is not None:
+            self.control.setShortcut(action.accelerator)
 
 
 class _Tool(HasTraits):
@@ -226,6 +242,9 @@ class _Tool(HasTraits):
     # group).
     group = Any
 
+    # The toolkit control.
+    control = Any()
+
     ###########################################################################
     # 'object' interface.
     ###########################################################################
@@ -244,8 +263,9 @@ class _Tool(HasTraits):
         if action.image is None:
             self.control = tool_bar.addAction(action.name)
         else:
-            self.control = tool_bar.addAction(action.image.create_icon(),
-                    action.name)
+            size = tool_bar.iconSize()
+            image = action.image.create_icon((size.width(), size.height()))
+            self.control = tool_bar.addAction(image, action.name)
 
         QtCore.QObject.connect(self.control, QtCore.SIGNAL('triggered()'),
                                self._qt4_on_triggered)
@@ -281,6 +301,9 @@ class _Tool(HasTraits):
         action.on_trait_change(self._on_action_visible_changed, 'visible')
         action.on_trait_change(self._on_action_checked_changed, 'checked')
 
+        # Detect if the control is destroyed.
+        self.control.destroyed.connect(self._qt4_on_destroyed)
+
         if controller is not None:
             self.controller = controller
             controller.add_to_toolbar(self)
@@ -289,6 +312,12 @@ class _Tool(HasTraits):
     # Private interface.
     ###########################################################################
 
+    def _qt4_on_destroyed(self, control=None):
+        """ Delete the reference to the control to avoid attempting to talk to
+        it again.
+        """
+        self.control = None
+
     def _qt4_on_triggered(self):
         """ Called when the tool bar tool is clicked. """
 
@@ -297,12 +326,7 @@ class _Tool(HasTraits):
 
         # Perform the action!
         if self.controller is not None:
-            # fixme: There is a difference here between having a controller
-            # and not in that in this case we do not set the checked state
-            # of the action! This is confusing if you start off without a
-            # controller and then set one as the action now behaves
-            # differently!
-            self.checked = self.control.isChecked()
+            self.checked = action.checked = self.control.isChecked()
 
             # Most of the time, action's do no care about the event (it
             # contains information about the time the event occurred etc), so
@@ -319,7 +343,7 @@ class _Tool(HasTraits):
                 self.controller.perform(action, action_event)
 
         else:
-            action.checked = self.control.isChecked()
+            self.checked = action.checked = self.control.isChecked()
 
             # Most of the time, action's do no care about the event (it
             # contains information about the time the event occurred etc), so
@@ -337,33 +361,33 @@ class _Tool(HasTraits):
 
     def _enabled_changed(self):
         """ Called when our 'enabled' trait is changed. """
-
-        self.control.setEnabled(self.enabled)
+        if self.control is not None:
+            self.control.setEnabled(self.enabled)
 
     def _visible_changed(self):
         """ Called when our 'visible' trait is changed. """
-
-        self.control.setVisible(self.visible)
+        if self.control is not None:
+            self.control.setVisible(self.visible)
 
     def _checked_changed(self):
         """ Called when our 'checked' trait is changed. """
-
-        self.control.setChecked(self.checked)
+        if self.control is not None:
+            self.control.setChecked(self.checked)
 
     def _on_action_enabled_changed(self, action, trait_name, old, new):
         """ Called when the enabled trait is changed on an action. """
-
-        self.control.setEnabled(action.enabled)
+        if self.control is not None:
+            self.control.setEnabled(action.enabled)
 
     def _on_action_visible_changed(self, action, trait_name, old, new):
         """ Called when the visible trait is changed on an action. """
-
-        self.control.setVisible(action.visible)
+        if self.control is not None:
+            self.control.setVisible(action.visible)
 
     def _on_action_checked_changed(self, action, trait_name, old, new):
         """ Called when the checked trait is changed on an action. """
-
-        self.control.setChecked(action.checked)
+        if self.control is not None:
+            self.control.setChecked(action.checked)
 
 
 class _PaletteTool(HasTraits):
