@@ -10,8 +10,10 @@ from traits.api import DelegatesTo, Instance, implements, on_trait_change
 # Local imports.
 from pyface.tasks.i_advanced_editor_area_pane import IAdvancedEditorAreaPane
 from pyface.tasks.i_editor_area_pane import MEditorAreaPane
+from editor_area_pane import EditorAreaDropFilter
 from main_window_layout import MainWindowLayout, PaneItem
 from task_pane import TaskPane
+from util import set_focus
 
 ###############################################################################
 # 'AdvancedEditorAreaPane' class.
@@ -38,15 +40,26 @@ class AdvancedEditorAreaPane(TaskPane, MEditorAreaPane):
             pane.
         """
         self.control = control = EditorAreaWidget(self, parent)
+        self._filter = EditorAreaDropFilter(self)
+        self.control.installEventFilter(self._filter)
 
         # Add shortcuts for scrolling through tabs.
-        mod = 'Meta+' if sys.platform == 'darwin' else 'Alt+'
-        shortcut = QtGui.QShortcut(QtGui.QKeySequence(mod+'n'), self.control)
+        if sys.platform == 'darwin':
+            next_seq = 'Ctrl+}'
+            prev_seq = 'Ctrl+{'
+        elif sys.platform.startswith('linux'):
+            next_seq = 'Ctrl+PgDown'
+            prev_seq = 'Ctrl+PgUp'
+        else:
+            next_seq = 'Alt+n'
+            prev_seq = 'Alt+p'
+        shortcut = QtGui.QShortcut(QtGui.QKeySequence(next_seq), self.control)
         shortcut.activated.connect(self._next_tab)
-        shortcut = QtGui.QShortcut(QtGui.QKeySequence(mod+'p'), self.control)
+        shortcut = QtGui.QShortcut(QtGui.QKeySequence(prev_seq), self.control)
         shortcut.activated.connect(self._previous_tab)
 
         # Add shortcuts for switching to a specific tab.
+        mod = 'Ctrl+' if sys.platform == 'darwin' else 'Alt+'
         mapper = QtCore.QSignalMapper(self.control)
         mapper.mapped.connect(self._activate_tab)
         for i in xrange(1, 10):
@@ -58,10 +71,14 @@ class AdvancedEditorAreaPane(TaskPane, MEditorAreaPane):
     def destroy(self):
         """ Destroy the toolkit-specific control that represents the pane.
         """
+        self.control.removeEventFilter(self._filter)
+        self._filter = None
+        
         for editor in self.editors:
             editor_widget = editor.control.parent()
             self.control.destroy_editor_widget(editor_widget)
             editor.editor_area = None
+
         super(AdvancedEditorAreaPane, self).destroy()
 
     ###########################################################################
@@ -106,8 +123,9 @@ class AdvancedEditorAreaPane(TaskPane, MEditorAreaPane):
     def set_layout(self, layout):
         """ Applies a LayoutItem to the editors in the pane.
         """
-        self._main_window_layout.set_layout_for_area(
-            layout, QtCore.Qt.LeftDockWidgetArea)
+        if layout is not None:
+            self._main_window_layout.set_layout_for_area(
+                layout, QtCore.Qt.LeftDockWidgetArea)
 
     ###########################################################################
     # Private interface.
@@ -156,7 +174,11 @@ class AdvancedEditorAreaPane(TaskPane, MEditorAreaPane):
     @on_trait_change('editors:[dirty, name]')
     def _update_label(self, editor, name, new):
         editor.control.parent().update_title()
-        
+
+    @on_trait_change('editors:tooltip')
+    def _update_tooltip(self, editor, name, new):
+        editor.control.parent().update_tooltip()
+
 
 ###############################################################################
 # Auxillary classes.
@@ -224,9 +246,11 @@ class EditorAreaWidget(QtGui.QMainWindow):
 
         # Configure the QMainWindow.
         # FIXME: Currently animation is not supported.
+        self.setAcceptDrops(True)
         self.setAnimated(False)
         self.setDockNestingEnabled(True)
         self.setDocumentMode(True)
+        self.setFocusPolicy(QtCore.Qt.StrongFocus)
         self.setTabPosition(QtCore.Qt.AllDockWidgetAreas,
                             QtGui.QTabWidget.North)
 
@@ -308,16 +332,20 @@ class EditorAreaWidget(QtGui.QMainWindow):
         # Get the tabs in this editor's dock area before removing it.
         tabified = self.tabifiedDockWidgets(editor_widget)
         if tabified:
-            tabified = [tabified[0]] + self.tabifiedDockWidgets(tabified[0])
+            widgets = self.get_dock_widgets_ordered()
+            tabified = [widget for widget in widgets \
+                        if widget in tabified or widget == editor_widget]
         visible = self.get_dock_widgets_ordered(visible_only=True)
 
-        # Destroy and remove the editor.
+        # Destroy and remove the editor. Get the active widget first, since it
+        # may be destroyed!
+        next_widget = self.editor_area.active_editor.control.parent()
         self.destroy_editor_widget(editor_widget)
-        
+
         # Ensure that the appropriate editor is activated.
         editor_area = self.editor_area
         choices = tabified if len(tabified) >= 2 else visible
-        if len(choices) >= 2:
+        if len(choices) >= 2 and editor_widget == next_widget:
              i = choices.index(editor_widget)
              next_widget = choices[i+1] if i+1 < len(choices) else choices[i-1]
              editor_area.activate_editor(next_widget.editor)
@@ -372,6 +400,7 @@ class EditorAreaWidget(QtGui.QMainWindow):
                 child.currentChanged.connect(self._tab_index_changed,
                                              QtCore.Qt.UniqueConnection)
                 child.setTabsClosable(True)
+                child.setUsesScrollButtons(True)
                 child.tabCloseRequested.connect(self._tab_close_requested,
                                                 QtCore.Qt.UniqueConnection)
                 # FIXME: We would like to have the tabs movable, but this
@@ -446,8 +475,25 @@ class EditorAreaWidget(QtGui.QMainWindow):
                 QtCore.QCoreApplication.sendEvent(self._drag_widget, event)
                 return True
 
+        elif event.type() == QtCore.QEvent.ToolTip:
+            # QDockAreaLayout forces the tooltips to be QDockWidget.windowTitle,
+            # so we provide the tooltips manually.
+            widgets = self.get_dock_widgets_for_bar(tab_bar)
+            index = tab_bar.tabAt(event.pos())
+            tooltip = widgets[index].editor.tooltip if index >= 0 else ''
+            if tooltip:
+                QtGui.QToolTip.showText(event.globalPos(), tooltip, tab_bar)
+            return True
+
         return False
 
+    def focusInEvent(self, event):
+        """ Assign focus to the active editor, if possible.
+        """
+        active_editor = self.editor_area.active_editor
+        if active_editor:
+            set_focus(active_editor.control)
+
     ###########################################################################
     # Signal handlers.
     ###########################################################################
@@ -503,7 +549,12 @@ class EditorWidget(QtGui.QDockWidget):
         
         title_bar = self.titleBarWidget()
         if isinstance(title_bar, EditorTitleBarWidget):
-            title_bar.update_title()
+            title_bar.setTabText(0, title)
+
+    def update_tooltip(self):
+        title_bar = self.titleBarWidget()
+        if isinstance(title_bar, EditorTitleBarWidget):
+            title_bar.setTabToolTip(0, self.editor.tooltip)
 
     def update_title_bar(self):
         if self not in self.parent()._tear_widgets:
@@ -528,15 +579,13 @@ class EditorTitleBarWidget(QtGui.QTabBar):
     def __init__(self, editor_widget):
         super(EditorTitleBarWidget, self).__init__(editor_widget)
         self.addTab(editor_widget.windowTitle())
+        self.setTabToolTip(0, editor_widget.editor.tooltip)
         self.setDocumentMode(True)
         self.setExpanding(False)
         self.setTabsClosable(True)
 
         self.tabCloseRequested.connect(editor_widget.editor.close)
 
-    def update_title(self):
-        self.setTabText(0, self.parent().windowTitle())
-
     def mousePressEvent(self, event):
         self.parent().parent()._drag_widget = self.parent()
         event.ignore()
