@@ -5,6 +5,9 @@ from traits.api import Bool, Property, implements
 # System library imports.
 from pyface.qt import QtGui
 
+# Local imports.
+from util import set_focus
+
 
 class TaskPane(MTaskPane):
     """ The toolkit-specific implementation of a TaskPane.
@@ -14,10 +17,6 @@ class TaskPane(MTaskPane):
 
     implements(ITaskPane)
 
-    #### 'ITaskPane' interface ################################################
-
-    has_focus = Property(Bool)
-
     ###########################################################################
     # 'ITaskPane' interface.
     ###########################################################################
@@ -40,13 +39,4 @@ class TaskPane(MTaskPane):
         """ Gives focus to the control that represents the pane.
         """
         if self.control is not None:
-            self.control.setFocus()
-
-    ###########################################################################
-    # Private interface.
-    ###########################################################################
-
-    def _get_has_focus(self):
-        if self.control is not None:
-            return self.control.hasFocus()
-        return False
+            set_focus(self.control)
