@@ -11,8 +11,9 @@
 import sys
 
 # Major library imports.
-import sip
-from pyface.qt import QtCore, QtGui
+from pyface.qt import QtCore, QtGui, qt_api
+
+from pyface.image_resource import ImageResource
 
 
 class SplitTabWidget(QtGui.QSplitter):
@@ -28,6 +29,8 @@ class SplitTabWidget(QtGui.QSplitter):
 
     # The different hotspots of a QTabWidget.  An non-negative value is a tab
     # index and the hotspot is to the left of it.
+
+    tabTextChanged = QtCore.Signal(QtGui.QWidget, unicode)
     _HS_NONE = -1
     _HS_AFTER_LAST_TAB = -2
     _HS_NORTH = -3
@@ -43,7 +46,7 @@ class SplitTabWidget(QtGui.QSplitter):
 
         self.clear()
 
-        QtCore.QObject.connect(QtGui.qApp,
+        QtCore.QObject.connect(QtGui.QApplication.instance(),
                 QtCore.SIGNAL('focusChanged(QWidget *,QWidget *)'),
                 self._focus_changed)
 
@@ -174,7 +177,7 @@ class SplitTabWidget(QtGui.QSplitter):
             ch = _TabWidget(self)
             self.addWidget(ch)
 
-        idx = ch.addTab(w, text)
+        idx = ch.insertTab(self._current_tab_idx+1, w, text)
 
         # If the tab has been added to the current tab widget then make it the
         # current tab.
@@ -289,14 +292,18 @@ class SplitTabWidget(QtGui.QSplitter):
         # It is possible for the C++ layer of this object to be deleted between
         # the time when the focus change signal is emitted and time when the
         # slots are dispatched by the Qt event loop. This may be a bug in PyQt4.
-        if sip.isdeleted(self):
-            return
+        if qt_api == 'pyqt':
+            import sip
+            if sip.isdeleted(self):
+                return
 
         if self._repeat_focus_changes:
             self.emit(QtCore.SIGNAL('focusChanged(QWidget *,QWidget *)'),
                       old, new)
 
-        if isinstance(new, _DragableTabBar):
+        if new is None:
+            return
+        elif isinstance(new, _DragableTabBar):
             ntw = new.parent()
             ntidx = ntw.currentIndex()
         else:
@@ -416,7 +423,7 @@ class SplitTabWidget(QtGui.QSplitter):
         if dhs == self._HS_OUTSIDE:
             # Disable tab tear-out for now. It works, but this is something that
             # should be turned on manually. We need an interface for this.
-            #ticon, ttext, ttextcolor, twidg = self._remove_tab(stab_w, stab)
+            #ticon, ttext, ttextcolor, tbuttn, twidg = self._remove_tab(stab_w, stab)
             #self.new_window_request.emit(pos, twidg)
             return
 
@@ -430,9 +437,9 @@ class SplitTabWidget(QtGui.QSplitter):
                 if dhs == self._HS_AFTER_LAST_TAB and stab == stab_w.count()-1:
                     return
 
-            QtGui.qApp.blockSignals(True)
+            QtGui.QApplication.instance().blockSignals(True)
 
-            ticon, ttext, ttextcolor, twidg = self._remove_tab(stab_w, stab)
+            ticon, ttext, ttextcolor, tbuttn, twidg = self._remove_tab(stab_w, stab)
 
             if dhs == self._HS_AFTER_LAST_TAB:
                 idx = dtab_w.addTab(twidg, ticon, ttext)
@@ -451,6 +458,8 @@ class SplitTabWidget(QtGui.QSplitter):
                 idx = dtab_w.insertTab(dhs, twidg, ticon, ttext)
                 dtab_w.tabBar().setTabTextColor(idx, ttextcolor)
 
+            if (tbuttn):
+                dtab_w.show_button(idx)
             dsplit_w._set_current_tab(dtab_w, idx)
 
         else:
@@ -458,14 +467,16 @@ class SplitTabWidget(QtGui.QSplitter):
             if stab_w is dtab_w and stab_w.count() == 1:
                 return
 
-            QtGui.qApp.blockSignals(True)
+            QtGui.QApplication.instance().blockSignals(True)
 
             # Remove the tab from its current tab widget and create a new one
             # for it.
-            ticon, ttext, ttextcolor, twidg = self._remove_tab(stab_w, stab)
+            ticon, ttext, ttextcolor, tbuttn, twidg = self._remove_tab(stab_w, stab)
             new_tw = _TabWidget(dsplit_w)
-            new_tw.addTab(twidg, ticon, ttext)
+            idx = new_tw.addTab(twidg, ticon, ttext)
             new_tw.tabBar().setTabTextColor(0, ttextcolor)
+            if tbuttn:
+                new_tw.show_button(idx)
 
             # Get the splitter containing the destination tab widget.
             dspl = dtab_w.parent()
@@ -487,7 +498,7 @@ class SplitTabWidget(QtGui.QSplitter):
         if dsplit_w != self:
             self.tab_window_changed.emit(twidg)
 
-        QtGui.qApp.blockSignals(False)
+        QtGui.QApplication.instance().blockSignals(False)
 
     def _horizontal_split(self, spl, idx, hs):
         """ Returns a tuple of the splitter and index where the new tab widget
@@ -555,10 +566,11 @@ class SplitTabWidget(QtGui.QSplitter):
         icon = tab_w.tabIcon(tab)
         text = tab_w.tabText(tab)
         text_color = tab_w.tabBar().tabTextColor(tab)
+        button = tab_w.tabBar().tabButton(tab, QtGui.QTabBar.LeftSide)
         w = tab_w.widget(tab)
         tab_w.removeTab(tab)
 
-        return (icon, text, text_color, w)
+        return (icon, text, text_color, button, w)
 
     def _hotspot(self, pos):
         """ Return a tuple of the tab widget, hotspot and hostspot geometry (as
@@ -568,7 +580,7 @@ class SplitTabWidget(QtGui.QSplitter):
         miss = (None, self._HS_NONE, None)
 
         # Get the bounding rect of the cloned QTbarBar.
-        top_widget = QtGui.qApp.topLevelAt(global_pos)
+        top_widget = QtGui.QApplication.instance().topLevelAt(global_pos)
         if isinstance(top_widget, QtGui.QTabBar):
             cloned_rect = top_widget.frameGeometry()
         else:
@@ -577,7 +589,7 @@ class SplitTabWidget(QtGui.QSplitter):
         # Determine which visible SplitTabWidget, if any, is under the cursor
         # (compensating for the cloned QTabBar that may be rendered over it).
         split_widget = None
-        for top_widget in QtGui.qApp.topLevelWidgets():
+        for top_widget in QtGui.QApplication.instance().topLevelWidgets():
             for split_widget in top_widget.findChildren(SplitTabWidget, None):
                 visible_region = split_widget.visibleRegion()
                 widget_pos = split_widget.mapFromGlobal(global_pos)
@@ -726,6 +738,8 @@ class _TabWidget(QtGui.QTabWidget):
     # The active icon.  It is created when it is first needed.
     _active_icon = None
 
+    _spinner_data = None
+
     def __init__(self, root, *args):
         """ Initialise the instance. """
 
@@ -745,6 +759,24 @@ class _TabWidget(QtGui.QTabWidget):
         self.setTabsClosable(True)
         self.tabCloseRequested.connect(self._close_tab)
 
+        if not (_TabWidget._spinner_data):
+            _TabWidget._spinner_data = ImageResource('spinner.gif')
+
+    def show_button(self, index):
+        lbl = QtGui.QLabel(self)
+        movie = QtGui.QMovie(_TabWidget._spinner_data.absolute_path, parent=lbl)
+        movie.setCacheMode(QtGui.QMovie.CacheAll)
+        movie.setScaledSize(QtCore.QSize(16, 16))
+        lbl.setMovie(movie)
+        movie.start()
+        self.tabBar().setTabButton(index, QtGui.QTabBar.LeftSide, lbl)
+
+    def hide_button(self, index):
+        curr = self.tabBar().tabButton(index, QtGui.QTabBar.LeftSide)
+        if curr:
+            curr.close()
+            self.tabBar().setTabButton(index, QtGui.QTabBar.LeftSide, None)
+
     def active_icon(self):
         """ Return the QIcon to be used to indicate an active tab page. """
 
@@ -816,6 +848,11 @@ class _TabWidget(QtGui.QTabWidget):
 
         self._root._close_tab_request(self.widget(index))
 
+class _IndependentLineEdit(QtGui.QLineEdit):
+    def keyPressEvent(self, e):
+        QtGui.QLineEdit.keyPressEvent(self, e)
+        if (e.key() == QtCore.Qt.Key_Escape):
+            self.hide()
 
 class _DragableTabBar(QtGui.QTabBar):
     """ The _DragableTabBar class is a QTabBar that can be dragged around. """
@@ -831,6 +868,18 @@ class _DragableTabBar(QtGui.QTabBar):
 
         self._root = root
         self._drag_state = None
+        # LineEdit to change tab bar title
+        te = _IndependentLineEdit("", self)
+        te.hide()
+        te.connect(te, QtCore.SIGNAL('editingFinished()'), te, QtCore.SLOT('hide()'))
+        self.connect(te, QtCore.SIGNAL('returnPressed()'), self._setCurrentTabText)
+        self._title_edit = te
+
+    def resizeEvent(self, e):
+        # resize edit tab
+        if self._title_edit.isVisible():
+            self._resize_title_edit_to_current_tab()
+        QtGui.QTabBar.resizeEvent(self, e)
 
     def keyPressEvent(self, e):
         """ Reimplemented to handle traversal across different tab widgets. """
@@ -842,6 +891,14 @@ class _DragableTabBar(QtGui.QTabBar):
         else:
             e.ignore()
 
+    def mouseDoubleClickEvent(self, e):
+        self._resize_title_edit_to_current_tab()
+        te = self._title_edit
+        te.setText(self.tabText(self.currentIndex())[1:])
+        te.setFocus()
+        te.selectAll()
+        te.show()
+
     def mousePressEvent(self, e):
         """ Reimplemented to handle mouse press events. """
 
@@ -896,6 +953,8 @@ class _DragableTabBar(QtGui.QTabBar):
         QtGui.QTabBar.mouseReleaseEvent(self, e)
 
         if e.button() != QtCore.Qt.LeftButton:
+            if e.button() == QtCore.Qt.MidButton:
+                self.tabCloseRequested.emit(self.tabAt(e.pos()))
             return
 
         if self._drag_state is not None and self._drag_state.dragging:
@@ -913,6 +972,19 @@ class _DragableTabBar(QtGui.QTabBar):
 
         return -1
 
+    def _setCurrentTabText(self):
+        idx = self.currentIndex()
+        text = self._title_edit.text()
+        self.setTabText(idx, u'\u25b6'+text)
+        self._root.emit(QtCore.SIGNAL('tabTextChanged(QWidget *, QString)'), self.parent().widget(idx), text)
+
+    def _resize_title_edit_to_current_tab(self):
+        idx = self.currentIndex()
+        tab = QtGui.QStyleOptionTabV3()
+        self.initStyleOption(tab, idx)
+        rect = self.style().subElementRect(QtGui.QStyle.SE_TabBarTabText, tab)
+        self._title_edit.setGeometry(rect.adjusted(0,8,0,-8))
+
 
 class _DragState(object):
     """ The _DragState class handles most of the work when dragging a tab. """
