@@ -31,7 +31,7 @@ from wx import TheClipboard
 from pyface.api import Sorter, Widget
 from pyface.timer.api import do_later
 from traits.api import Bool, Color, Enum, Event, Font, Instance, Int, \
-     List, Trait
+     List, Trait, Undefined
 from pyface.wx.drag_and_drop import PythonDropSource, \
      PythonDropTarget, PythonObject
 from pyface.wx.drag_and_drop import clipboard as enClipboard
@@ -360,6 +360,11 @@ class Grid(Widget):
         otc(self._on_show_row_headers_changed, 'show_row_headers',
             remove = True)
 
+        # It seems that the grid must be destroyed before disposing of
+        # _grid_table_base: otherwise, the grid can apparently generate an
+        # extra _GridTableBase.GetAttr call after _GridTableBase.dispose()
+        # has been called, leading to headaches and segfaults.
+        grid.Destroy()
         self._grid_table_base.dispose()
 
     ###########################################################################
@@ -1703,9 +1708,10 @@ class _GridTableBase(PyGridTableBase):
                         self.model.is_column_read_only(col)
 
         result.SetReadOnly(read_only)
-        read_only_color = self._grid.default_cell_read_only_color
-        if read_only and read_only_color is not None:
-            result.SetBackgroundColour(read_only_color)
+        if read_only :
+            read_only_color = self._grid.default_cell_read_only_color
+            if read_only_color is not None and read_only_color is not Undefined:
+                result.SetBackgroundColour(read_only_color)
 
         # check to see if colors or fonts are specified for this cell
         bgcolor = None
